package de.uka.ipd.consensus.foundation.user;

import java.util.EventListener;

/**
 * The listener interface for receiving events that signal a change in the user database.
 * @author Thomas Much
 * @version 2006-06-14
 * @see de.uka.ipd.consensus.foundation.user.UserManager#addUserListener(UserListener)
 */
public interface UserListener extends EventListener {

	/**
	 * Called by the UserManager when a user logged on.
	 * @param event the UserEvent object containing the user who logged on
	 */
	public void userLoggedIn(UserEvent event);
	
	
	/**
	 * Called by the UserManager when a user logged off.
	 * @param event the UserEvent object containing the user who logged off
	 */
	public void userLoggedOut(UserEvent event);
	
	
	/**
	 * Called by the UserManager when a new user account has been created
	 * @param event the UserEvent object containing the new user account
	 */
	public void userCreated(UserEvent event);

}
